/* ControlPanel class */

package ciips.animation;

import java.awt.*;
import java.applet.*;
import java.io.*;
import java.net.*;
import java.util.*;


/**
 * This class extends the <code>java.awt.Panel</code> class to form
 * a panel which holds the buttons that control
 * the animation of the algorithm.
 * <p>
 * In this particular application, it only consists of four image buttons,
 * namely, <code>run, stop, step,</code> and <code>skip</code>.
 *
 * @see Panel
 */
public class ControlPanel extends Panel {
		private AlgAnimFrame frame;
		private Button control_buttons[];
		private Button runButton; 
		private Button stopButton;
		private Button stepButton, skipButton;
		private Font font;

		private static final String[] button_names = {
					"stop", "run", "step", "skip" };
		private static final int STOP_BTN = 0;
		private static final int RUN_BTN = 1;
		private static final int STEP_BTN = 2;
		private static final int SKIP_BTN = 3;

		/**
		 * Creates a panel to hold the control buttons of the animation
		 * tool.
		 * @param frame The parent window frame that contains this panel
		 * @param algname The algorithm name parsed from the applet parameter
		 */
		public ControlPanel(AlgAnimFrame frame, String algname) {
			setLayout( new FlowLayout(FlowLayout.LEFT) );
			this.frame = frame;
			final Font helv10 = frame.helv10;

			setBackground(Color.white);
			control_buttons = new Button[ button_names.length ];
			for( int k=0;k<button_names.length;k++) {	
				control_buttons[k] = new ImageButton(button_names[k], frame.getApplet(), this);
				this.add( control_buttons[k] );
				}
			stopButton = control_buttons[STOP_BTN];
			runButton = control_buttons[RUN_BTN];
			stepButton = control_buttons[STEP_BTN];
			skipButton = control_buttons[SKIP_BTN];
			((ImageButton)stopButton).setDisable();
			// this.add(stopButton);
			/*
			runButton = new ImageButton("run", frame.getApplet(), this);
			this.add(runButton);
	
			stepButton = new ImageButton("step", frame.getApplet(), this);
			this.add(stepButton);

			skipButton = new ImageButton("skip", frame.getApplet(), this);
			this.add(skipButton);
			*/
			refreshButtons();
			} // ControlPanel()

		/**
		 * Perform a <code>repaint()</code> on each of the image buttons.
		 */
		public void refreshButtons() {
			for(int k=0;k<button_names.length;k++) {
			 if (control_buttons[k] != null)
					control_buttons[k].repaint();
				}
			/* if (stopButton != null)
					stopButton.repaint();
			if (stepButton != null)
					stepButton.repaint();
			if (skipButton != null)
					skipButton.repaint(); */
			}

		/**
		 * Action handler for the buttons and choice buttons in the control
		 * panel.
		 * @param e Event invoked
		 * @param arg Object that invokes the event
		 */
	public boolean action(Event e, Object arg) {
				Object target = e.target;
				System.out.println("ControlPanel:action - target " + target);
 
				refreshButtons();
				if (target == runButton) {
					((ImageButton)skipButton).setDisable();
					((ImageButton)runButton).setDisable();
					frame.setSkip(false);
					frame.setText(0, "Running continuously...");
					frame.startAlg();
					} 
				else if (target == stopButton) {
					frame.setText(0, "Animation stopped");
					frame.setStep(false);
					if (frame.getAlg().isAlive())
								frame.getAlg().stop();
					frame.finishAlg();
					((ImageButton)skipButton).setEnable();
					}
				else if (target == stepButton) {
					System.out.println("ControlPanel:action - step");
					frame.setSkip( false );
					((ImageButton)skipButton).setEnable();
					frame.setStep( true );
					if (!frame.getAlg().isAlive())
						frame.startAlg();
					// else
						//frame.getAlg().resume();
					((ImageButton)stepButton).setDisable();
					}
				else if (target == skipButton) {
					frame.setSkip(true);
					frame.startAlg();
					frame.setText(0, "Skipping to the next phase...");
					((ImageButton)skipButton).setDisable();
					} 
				refreshButtons();
				return true;
				} // action()

		/**
		 * Returns an instance of the run <code>ImageButton</code>, which is
		 * cast to <code>java.awt.Button</code>
		 * @return Run button to initiate the animation of the algorithm.
		 */
	public Button getRunButton() {
		return control_buttons[RUN_BTN];
		}

		/**
		 * Returns an instance of the stop <code>ImageButton</code>, which is
		 * cast to <code>java.awt.Button</code>
		 * @return Stop button to terminate the execution of algorithm.
		 */
	public Button getStopButton() {
		return control_buttons[STOP_BTN];
		}

		/**
		 * Returns an instance of the step <code>ImageButton</code>, which is
		 * casted to <code>java.awt.Button</code>
		 * @return Step button to trace through the execution of the algorithm.
		 * If the <code>Run</code> button has already been pressed, the
		 * execution mode will be switched from <code>RUN</code> to
		 * <code>STEP</code>.
		 */
	public Button getStepButton() {
		return control_buttons[STEP_BTN];
		}

		/**
		 * Returns an instance of the skip <code>ImageButton</code>, which is
		 * casted to <code>java.awt.Button</code>
		 * @return Skip button to bypass the animation of the algorithm.
		 */
		public Button getSkipButton() {
			return control_buttons[SKIP_BTN];
		}
} // class ControlPanel
