
package ciips.animation.tree;

import java.awt.*;
import ciips.animation.*;

/**
 * This class regards a TreeNode as a binary tree.
 * Its <TT>draw</TT> method draws the descending arcs also.
 * @see TreeNode
 * @see DrawingPanel#addDrawingObj
 * @see DrawingPanel#removeObj
 */
public class MultiwayTree extends MultiNode /* implements DrawingObj*/ {
    public int size;


    /**
     * Create a new tree
     */
    public MultiwayTree() {
	size=getChildCount();
    } // Constructor 1
    
    /** Start at a node and set the positions for the sub-tree elements 
     *   
     **/
    public void setPosition( int x, int y, int dx, int dy  ) {
	this.x = x; this.y = y;
	int dx2 = dx/2;
	for (int i=0; i<size; i++) {
	    if (child[i]!=null){
		child[i].setPosition(((x-dx2)+(i*dx/size)), y+dy, dx2, dy);
	    }
	}
    }

    
    
    /**
     * Sets the depth of this node corresponding to the root node of the tree.
     * @param depth Depth of the node.
     */
    public void setDepth(int depth) {
	this.depth = depth;
    }
    
    /**
     * Move the node and all its branches based on the parameters.
     * @param x The horizontal destination position of this node.
     * @param y The vertical destination position of this node.
     */
    public void move(int x, int y) {
			int dx = x - this.x;
			int dy = y - this.y;
			displace(dx, dy);
    	}
    
    /**
     * Displace the tree starting with node dx pixels to the right and dy
     * pixels down.
     * @param node The root node of the tree to be moved.
     * @param dx The change in x direction.
     * @param dy The change in y direction.
     */
    public void displace(int dx, int dy) {
			super.displace(dx,dy);
			if (!isLeaf()) {
	    	for (int i=0; i<size; i++){
					if (getTreeNode(i) != null)
		    		getTreeNode(i).displace(dx,dy);
	    		}
				}
    	}
    
    
    /** Draw a node and its subtree */
    public void drawSubTree( Graphics g ) {
	draw( g ) ;
	for (int i=0; i<size ; i++) {
	    if (child[i]!=null) child[i].drawSubTree( g );
	}
    }
    
    public static void drawEdge( Graphics g, MultiNode parent, MultiNode child, boolean highlight ) {
	g.setColor( Color.black );
	g.drawLine(parent.x + 6, parent.y + 30, child.x + 10, child.y);
	if (highlight) {
	    g.setColor( Color.blue );
	    g.drawLine(parent.x + 5, parent.y + 30, child.x + 9, child.y);
	    g.drawLine(parent.x + 4, parent.y + 30, child.x + 8, child.y);
	}
    }
    
    /**
     * This method draws the node on the corresponding graphical context
     * normally passed in from the drawing panel.
     */
    public void draw(Graphics g) {
	System.out.println("MultiwayTreeNode:draw " + getData() );
	if (highlight)
	    g.setColor(Color.black);
	else
	    g.setColor( nodeColor );
	g.fillRect(x, y, 20, 30);
	g.setColor(Color.black);
	g.drawRect(x, y, 20, 30);
	if ( label != null ) {
	    if (getLabel().length() > 0) {
		g.setColor( labelColor );
		g.setFont( hugeFont );
		g.drawString(getLabel(), x + 5, y + 12);
	    }
	}
	g.setColor( Color.white );
	g.setFont( bigFont );
	g.drawString(""+getData(), x + 2, y+27);
	
	// draw links to children
	for (int i=0; i<size; i++) {
	    if (getTreeNode(i) != null) {
		drawEdge( g, this, child[i], highlightNode);
	    }
	}
	
	
	System.out.println("BinaryTree:draw " + getData() + " exit" );
    }
}

